/**********************************************************************/
/*                                                                    */
/* COMPONENT_NAME:  smprt (SMP Runtime Library)                       */
/* COMPONENT_ID:    (5799-AH1)                                        */
/*                                                                    */
/*   Licensed Materials - Property of IBM.                            */
/*   IBM XL C/C++ for Blue Gene, V12.1                                */
/*   (5799-AG1)                                                       */
/*   IBM XL Fortran for Blue Gene, V14.1                              */
/*   (5799-AH1)                                                       */
/*   Copyright IBM Corp. 1999, 2012.                                  */
/*   All Rights Reserved.                                             */
/*   US Government Users Restricted Rights -                          */
/*   Use, duplication or disclosure restricted by                     */
/*   GSA ADP Schedule Contract with IBM Corp.                         */
/*                                                                    */
/**********************************************************************/
/*  SCCS ID STRING    01  1.10.11.2 4/4/12 21:33:04        10/9/13 18:13:21  */

#ifdef __cplusplus
extern "C" {
#endif


#if !defined(_OMP_H_)
#define	_OMP_H_
/* The #endif for this is at the end of the file. */


#include <pthread.h>


typedef int omp_lock_t;

typedef struct {
  int       lock;
  int       count;
  pthread_t thread;
} omp_nest_lock_t;

typedef enum omp_sched {
  omp_sched_unknown= 0,/* not a valid type */

  /* following 4 schedule kinds are defined in Open 3.0 Specification */
  omp_sched_static= 1,/* GUIDED scheduling type */
  omp_sched_dynamic= 2,/* DYNAMIC scheduling type */
  omp_sched_guided= 3,/* STATIC scheduling type */
  omp_sched_auto= 4,/* AUTO scheduling type */

  ibm_sched_affinity= 1000/* AFFINITY scheduling type. This is an IBM extension. */

} omp_sched_t;


double omp_get_wtick(void);
double omp_get_wtime(void);

#if defined(_OPENMP) || defined(_IBMSMP) 

void omp_set_num_threads(int num);
int  omp_get_num_threads(void);
int  omp_get_max_threads(void);
int  omp_get_thread_limit(void);
int  omp_get_thread_num(void);
int  omp_get_num_procs(void);
int  omp_in_parallel(void);
int  omp_in_final(void);

void omp_set_dynamic(int flag);
int  omp_get_dynamic(void);

void omp_set_nested(int flag);
int  omp_get_nested(void);

void omp_set_max_active_levels(int level);
int omp_get_max_active_levels(void);

int omp_get_level(void);
int omp_get_active_level(void);
int omp_get_ancestor_thread_num(int level);
int omp_get_team_size(int level);

void omp_init_lock(omp_lock_t *lock);
void omp_init_nest_lock(omp_nest_lock_t *lock);

void omp_destroy_lock(omp_lock_t *lock);
void omp_destroy_nest_lock(omp_nest_lock_t *lock);

void omp_set_lock(omp_lock_t *lock);
void omp_set_nest_lock(omp_nest_lock_t *lock);

void omp_unset_lock(omp_lock_t *lock);
void omp_unset_nest_lock(omp_nest_lock_t *lock);

int  omp_test_lock(omp_lock_t *lock);
int  omp_test_nest_lock(omp_nest_lock_t *lock);

int omp_get_schedule(omp_sched_t * kind, int * modifier);
void omp_set_schedule(omp_sched_t kind, int modifier);

#else 

#if __cplusplus
#define __omp_inline  static inline
#else
#define __omp_inline  static __inline
#endif

__omp_inline void omp_set_schedule(omp_sched_t kind, int modifier) {
}

__omp_inline int omp_get_schedule(omp_sched_t * kind, int * modifier) {
  return omp_sched_static;
}

__omp_inline void omp_set_num_threads(int num) {
}

__omp_inline int omp_get_num_threads(void) {
  return 1;
}

__omp_inline int omp_get_max_threads(void) {
  return 1;
}

__omp_inline int omp_get_thread_limit(void) {
  return 1;
}

__omp_inline int omp_get_thread_num(void) {
  return 0;
}

__omp_inline int omp_get_num_procs(void) {
  return 1;
}

__omp_inline int omp_in_parallel(void) {
  return 0;
}

__omp_inline int omp_in_final(void) {
     return 0;
}

__omp_inline void omp_set_dynamic(int flag) {
}

__omp_inline int omp_get_dynamic(void) {
  return 0;
}

__omp_inline void omp_set_nested(int flag) {
}

__omp_inline int omp_get_nested(void) {
  return 0;
}

__omp_inline void omp_set_max_active_levels(int level) {
}

__omp_inline int omp_get_max_active_levels(int level) {
  return 1;
}

__omp_inline int omp_get_level(void) {
  return 0;
}

__omp_inline int omp_get_active_level(void) {
  return 0;
}

__omp_inline int omp_get_ancestor_thread_num(int level) {
  return 0;
}

__omp_inline int omp_get_team_size(int level) {
  return 1;
}

__omp_inline void omp_init_lock(omp_lock_t  *lock) {
}

__omp_inline void omp_init_nest_lock(omp_nest_lock_t *lock) {
  lock->count = 0;
}

__omp_inline void omp_destroy_lock(omp_lock_t *lock) {
}

__omp_inline void omp_destroy_nest_lock(omp_nest_lock_t *lock) {
}

__omp_inline void omp_set_lock(omp_lock_t *lock) {
}

__omp_inline void omp_set_nest_lock(omp_nest_lock_t *lock) {
  lock->count++;
}

__omp_inline void omp_unset_lock(omp_lock_t *lock) {
}

__omp_inline void omp_unset_nest_lock(omp_nest_lock_t *lock) {
  lock->count--;
}

__omp_inline int omp_test_lock(omp_lock_t *lock) {
  return 1;
}

__omp_inline int omp_test_nest_lock(omp_nest_lock_t *lock) {
  lock->count++;
  return lock->count;
}

#undef __omp_inline
#endif  /* (_OPENMP) || defined(_IBMSMP) */

#endif  /* !defined(_OMP_H_) */


#ifdef __cplusplus
}
#endif

