#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "probe.h"
#include "printing.h"

int32_t POMP_Init(void) {
    print("initializing POMP timer probe\n");

    init_timing();
}

void POMP_Finalize(void) {
    print("exiting\n");
}

int32_t POMP_Get_handle(POMP_Handle_t *handle, char ctc[]) {
    enum {
        MODE_SEPARATOR,  /* looking for and skipping a '*' */
        MODE_KEY,        /* looking for a key, ending with '=' */
        MODE_VALUE,      /* looking for a value, ending with '*' */
        MODE_INTERESTING_VALUE,  /* the value we're looking for */
        MODE_DONE        /* finished parsing */
    } mode = MODE_SEPARATOR;

    int length;
    char *start = ctc, *current = ctc;

    *handle = "unknown";  /* assume nothing found at the beginning */

    length = strtol(current, &current, 0);
    if(current && *current == '*') current ++;  /* skip first '*' */

    while(current && *current && mode != MODE_DONE) {
        switch(mode) {
        case MODE_SEPARATOR:
            if(*current == '*') {  /* two '*' in a row */
                mode = MODE_DONE;
            }
            else {
                start = current;
                mode = MODE_KEY;
            }
            break;
        case MODE_KEY:
            if(*current == '=') {
                if(current - start == 4 && strncmp(start, "sscl", 4) == 0) {
                    mode = MODE_INTERESTING_VALUE;
                }
                else mode = MODE_VALUE;

                current ++;
                start = current;
            }
            else current ++;
            break;
        case MODE_VALUE:
        case MODE_INTERESTING_VALUE:
            if(*current == '*') {
                if(mode == MODE_INTERESTING_VALUE) {
                    char *data = malloc(current - start + 1);
                    memcpy(data, start, current - start);
                    data[current - start] = 0;

                    *handle = data;
                }

                current ++;
                mode = MODE_SEPARATOR;
            }
            else current ++;
            break;
        default:
            break;
        }
    }
}


int32_t POMP_Parallel_enter(
    POMP_Handle_t *handle,
    int32_t thread_id,
    int32_t num_threads,
    int32_t if_expr_result,
    char ctc[]) {
    
    print_callback(*handle, thread_id,
        "enter parallel region with %d threads, if_expr_result=%d\n",
        num_threads, if_expr_result);
}

int32_t POMP_Parallel_begin(POMP_Handle_t *handle, int32_t thread_id) {
    print_callback(*handle, thread_id,
        "begin parallel region\n");
    add_indent(thread_id);
}

int32_t POMP_Parallel_end(POMP_Handle_t *handle, int32_t thread_id) {
    subtract_indent(thread_id);
    print_callback(*handle, thread_id,
        "end parallel region\n");
}

int32_t POMP_Parallel_exit(POMP_Handle_t *handle, int32_t thread_id) {
    print_callback(*handle, thread_id,
        "exit parallel region\n");
}


int32_t POMP_Loop_enter(
    POMP_Handle_t *handle,
    int32_t thread_id,
    int64_t chunk_size,
    int64_t init_iter,
    int64_t final_iter,
    int64_t incr,
    char ctc[]) {

    print_callback(*handle, thread_id,
        "enter loop [%ld,%ld) chunk size %ld\n",
        init_iter, final_iter, chunk_size);
    add_indent(thread_id);
}

int32_t POMP_Loop_chunk_begin(POMP_Handle_t *handle,
    int32_t thread_id,
    int64_t init_iter,
    int64_t final_iter) {

    print_callback(*handle, thread_id,
        "begin chunk [%d,%d)\n",
        init_iter, final_iter);
}

int32_t POMP_Loop_chunk_end(POMP_Handle_t *handle, int32_t thread_id) {
    print_callback(*handle, thread_id,
        "end chunk\n");
}

int32_t POMP_Loop_exit(POMP_Handle_t *handle, int32_t thread_id) {
    subtract_indent(thread_id);
    print_callback(*handle, thread_id,
        "exit loop\n");
}

